@@ -9,12 +9,11 @@ from slixmpp import JID, Message, Presence
 from slixmpp.exceptions import IqError, IqTimeout
 from slixmpp.plugins.xep_0292.stanza import VCard4
 from slixmpp.types import MessageTypes
-from sqlalchemy.exc import IntegrityError
 
 from ..core.mixins import AvatarMixin, FullCarbonMixin
 from ..core.mixins.disco import ContactAccountDiscoMixin
 from ..core.mixins.recipient import ReactionRecipientMixin, ThreadRecipientMixin
-from ..db.models import Contact, ContactSent, Participant
+from ..db.models import Contact, ContactSent
 from ..util import SubclassableOnce
 from ..util.types import ClientType, LegacyUserIdType, MessageOrPresenceTypeVar
 
@@ -75,6 +74,7 @@ class LegacyContact(
 
     mtype: MessageTypes = "chat"
     _can_send_carbon = True
+    is_participant = False
     is_group = False
 
     _ONLY_SEND_PRESENCE_CHANGES = True
@@ -139,7 +139,7 @@ class LegacyContact(
             participants = self.stored.participants
         for p in participants:
             with self.xmpp.store.session() as orm:
-                orm.add(p)
+                p = orm.merge(p)
                 muc = self.session.bookmarks.from_store(p.room)
                 yield muc.participant_from_store(p, contact=self)
 
@@ -241,32 +241,39 @@ class LegacyContact(
             and self.xmpp.MARK_ALL_MESSAGES
             and is_markable(stanza)
         ):
-            try:
-                with self.xmpp.store.session(expire_on_commit=False) as orm:
-                    self.stored = orm.merge(self.stored)
+            with self.xmpp.store.session(expire_on_commit=False) as orm:
+                self.stored = orm.merge(self.stored)
+                exists = (
+                    orm.query(ContactSent)
+                    .filter_by(contact_id=self.stored.id, msg_id=stanza["id"])
+                    .first()
+                )
+                if exists:
+                    self.log.warning(
+                        "Contact has already sent message %s", stanza["id"]
+                    )
+                else:
                     new = ContactSent(contact=self.stored, msg_id=stanza["id"])
                     orm.add(new)
                     self.stored.sent_order.append(new)
                     orm.commit()
-            except IntegrityError:
-                self.log.warning("Contact  has already sent message %s", stanza["id"])
         stanza["to"] = self.user_jid
         stanza.send()
         return stanza
 
-    def get_msg_xmpp_id_up_to(self, horizon_xmpp_id: str) -> list[str]:
+    def pop_unread_xmpp_ids_up_to(self, horizon_xmpp_id: str) -> list[str]:
         """
         Return XMPP msg ids sent by this contact up to a given XMPP msg id.
 
-        Plugins have no reason to use this, but it is used by slidge core
+        Legacy modules have no reason to use this, but it is used by slidge core
         for legacy networks that need to mark all messages as read (most XMPP
         clients only send a read marker for the latest message).
 
         This has side effects, if the horizon XMPP id is found, messages up to
-        this horizon are not cleared, to avoid sending the same read mark twice.
+        this horizon are cleared, to avoid sending the same read mark twice.
 
         :param horizon_xmpp_id: The latest message
-        :return: A list of XMPP ids or None if horizon_xmpp_id was not found
+        :return: A list of XMPP ids up to horizon_xmpp_id, included
         """
         with self.xmpp.store.session() as orm:
             assert self.stored.id is not None
@@ -323,6 +330,13 @@ class LegacyContact(
         locality: Optional[str] = None,
         pronouns: Optional[str] = None,
     ) -> None:
+        """
+        Update xep:`0292` data for this contact.
+
+        Use this for additional metadata about this contact to be available to XMPP
+        clients. The "note" argument is a text of arbitrary size and can be useful when
+        no other field is a good fit.
+        """
         vcard = VCard4()
         vcard.add_impp(f"xmpp:{self.jid.bare}")
 
@@ -387,11 +401,9 @@ class LegacyContact(
                 jid=self.user_jid, roster_items=self.get_roster_item()
             )
         except PermissionError:
-            from slidge import __version__
-
             warnings.warn(
-                "Slidge does not have the privilege to manage rosters. See "
-                f"https://slidge.im/docs/slidge/{__version__}/admin/privilege.html"
+                "Slidge does not have the privilege (XEP-0356) to manage rosters. "
+                "Consider configuring your XMPP server for that."
             )
             self.send_friend_request(
                 f"I'm already your friend on {self.xmpp.COMPONENT_TYPE}, but "
