@@ -68,8 +68,9 @@ DB_URL__CATEGORY = _Categories.ADVANCED
 USER_JID_VALIDATOR: str
 USER_JID_VALIDATOR__DOC = (
     "Regular expression to restrict users that can register to the gateway, by JID. "
-    "Defaults to .*@${SLIDGE_SERVER}, but since SLIDGE_SERVER is usually localhost, "
-    "you probably want to change that to .*@example.com"
+    "Defaults to .*@${INFERRED_SERVER}. INFERRED_SERVER is derived for the gateway JID, "
+    "by removing whatever is before the first encountered dot in it. Example: if "
+    "slidge's JID=slidge.example.org, INFERRED_SERVER=example.org."
 )
 USER_JID_VALIDATOR__DYNAMIC_DEFAULT = True
 USER_JID_VALIDATOR__CATEGORY = _Categories.BASE
@@ -104,10 +105,21 @@ USE_ATTACHMENT_ORIGINAL_URLS__CATEGORY = _Categories.ATTACHMENTS
 
 UPLOAD_REQUESTER: Optional[str] = None
 UPLOAD_REQUESTER__DOC = (
-    "Set which JID should request the upload slots. Defaults to the component JID."
+    "Set which JID should request the upload slots. Defaults to the user's JID if "
+    "IQ/get privileges granted for the 'urn:xmpp:http:upload:0' namespace; the component "
+    "JID otherwise."
 )
 UPLOAD_REQUESTER__CATEGORY = _Categories.ATTACHMENTS
 
+UPLOAD_URL_PREFIX: Optional[str] = None
+UPLOAD_URL_PREFIX__DOC = (
+    "This is an optional setting to make sure the URL of your upload service is never leaked "
+    "to the legacy network in bodies of messages. This can happen under rare circumstances and/or bugs,"
+    "when replying to an attachment. Set this to the common prefix of the public URL your attachments get, "
+    "eg https://upload.example.org:5281/"
+)
+UPLOAD_URL_PREFIX__CATEGORY = _Categories.ATTACHMENTS
+
 NO_UPLOAD_PATH: Optional[str] = None
 NO_UPLOAD_PATH__DOC = (
     "Instead of using the XMPP server's HTTP upload component, copy files to this dir. "
@@ -186,6 +198,12 @@ ATTACHMENT_MAXIMUM_FILE_NAME_LENGTH__DOC = (
 )
 ATTACHMENT_MAXIMUM_FILE_NAME_LENGTH__CATEGORY = _Categories.ATTACHMENTS
 
+CONVERT_STICKERS = False
+CONVERT_STICKERS__DOC = (
+    "Convert lottie vector stickers (from the legacy side) to webp animations."
+)
+CONVERT_STICKERS__CATEGORY = _Categories.ATTACHMENTS
+
 AVATAR_RESAMPLING_THREADS = 2
 AVATAR_RESAMPLING_THREADS__DOC = (
     "Number of additional threads to use for avatar resampling. Even in a single-core "
