@@ -25,17 +25,23 @@ class TextMessageMixin(MessageMaker):
         elif self.mtype == "groupchat":
             return {"markable"}
 
-    def _replace_id(self, legacy_msg_id: LegacyMessageType):
+    def _replace_id(self, legacy_msg_id: LegacyMessageType) -> str:
         if self.mtype == "groupchat":
             with self.xmpp.store.session() as orm:
-                ids = self.xmpp.store.id_map.get_xmpp(
-                    orm, self._recipient_pk(), str(legacy_msg_id), True
+                ids = self.xmpp.store.id_map.get_origin(
+                    orm, self._recipient_pk(), str(legacy_msg_id)
                 )
                 if ids:
+                    if len(ids) > 1:
+                        log.warning(
+                            "More than 1 origin msg ID for '%s': '%s'",
+                            legacy_msg_id,
+                            ids,
+                        )
                     return ids[0]
                 return self.session.legacy_to_xmpp_msg_id(legacy_msg_id)
         else:
-            return self._legacy_to_xmpp(legacy_msg_id)
+            return self._legacy_to_xmpp(legacy_msg_id)[0]
 
     def send_text(
         self,
@@ -76,27 +82,22 @@ class TextMessageMixin(MessageMaker):
         :param archive_only: (only in groups) Do not send this message to user,
             but store it in the archive. Meant to be used during ``MUC.backfill()``
         """
-        if carbon and not hasattr(self, "muc"):
+        if carbon and not self.is_participant:
             with self.xmpp.store.session() as orm:
                 if not correction and self.xmpp.store.id_map.was_sent_by_user(
-                    orm, self._recipient_pk(), str(legacy_msg_id), self.is_group
+                    orm, self._recipient_pk(), str(legacy_msg_id), True
                 ):
                     log.warning(
                         "Carbon message for a message an XMPP has sent? This is a bug! %s",
                         legacy_msg_id,
                     )
                     return
-                if hasattr(self, "muc") and not self.is_user:  # type:ignore
-                    log.warning(
-                        "send_text() called with carbon=True on a participant who is not the user",
-                        legacy_msg_id,
-                    )
                 self.xmpp.store.id_map.set_msg(
                     orm,
                     self._recipient_pk(),
                     str(legacy_msg_id),
                     [self.session.legacy_to_xmpp_msg_id(legacy_msg_id)],
-                    self.is_group,
+                    True,
                 )
                 orm.commit()
         hints = self.__default_hints(hints)
@@ -185,15 +186,20 @@ class TextMessageMixin(MessageMaker):
         :param emojis: An iterable of emojis used as reactions
         :param thread:
         """
-        msg = self._make_message(
-            hints={"store"}, carbon=bool(kwargs.get("carbon")), thread=thread
-        )
         xmpp_id = kwargs.pop("xmpp_id", None)
-        if not xmpp_id:
-            xmpp_id = self._legacy_to_xmpp(legacy_msg_id)
-        self.xmpp["xep_0444"].set_reactions(msg, to_id=xmpp_id, reactions=set(emojis))
-        self.__add_reaction_fallback(msg, legacy_msg_id, emojis)
-        self._send(msg, **kwargs)
+        if xmpp_id:
+            xmpp_ids = [xmpp_id]
+        else:
+            xmpp_ids = self._legacy_to_xmpp(legacy_msg_id)
+        for xmpp_id in xmpp_ids:
+            msg = self._make_message(
+                hints={"store"}, carbon=bool(kwargs.get("carbon")), thread=thread
+            )
+            self.xmpp["xep_0444"].set_reactions(
+                msg, to_id=xmpp_id, reactions=set(emojis)
+            )
+            self.__add_reaction_fallback(msg, legacy_msg_id, emojis)
+            self._send(msg, **kwargs)
 
     def __add_reaction_fallback(
         self,
@@ -231,18 +237,23 @@ class TextMessageMixin(MessageMaker):
         :param legacy_msg_id: Legacy ID of the message to delete
         :param thread:
         """
-        msg = self._make_message(
-            state=None,
-            hints={"store"},
-            mbody=f"/me retracted the message {legacy_msg_id}",
-            carbon=bool(kwargs.get("carbon")),
-            thread=thread,
-        )
-        msg.enable("fallback")
-        # namespace version mismatch between slidge and slixmpp, update me later
-        msg["fallback"]["for"] = self.xmpp["xep_0424"].namespace[:-1] + "1"
-        msg["retract"]["id"] = msg["replace"]["id"] = self._replace_id(legacy_msg_id)
-        self._send(msg, **kwargs)
+        xmpp_ids = self._legacy_to_xmpp(legacy_msg_id)
+        replace_id = self._replace_id(legacy_msg_id)
+        if replace_id not in xmpp_ids:
+            xmpp_ids.append(replace_id)
+        for xmpp_id in xmpp_ids:
+            msg = self._make_message(
+                state=None,
+                hints={"store"},
+                mbody=f"/me retracted the message {legacy_msg_id}",
+                carbon=bool(kwargs.get("carbon")),
+                thread=thread,
+            )
+            msg.enable("fallback")
+            # namespace version mismatch between slidge and slixmpp, update me later
+            msg["fallback"]["for"] = self.xmpp["xep_0424"].namespace[:-1] + "1"
+            msg["retract"]["id"] = msg["replace"]["id"] = xmpp_id
+            self._send(msg, **kwargs)
 
 
 log = logging.getLogger(__name__)
