@@ -559,7 +559,7 @@ class LegacyParticipant(
 
 
 def escape_nickname(muc_jid: JID, nickname: str) -> tuple[str, JID]:
-    nickname = nickname_no_illegal = strip_illegal_chars(nickname)
+    nickname = nickname_no_illegal = strip_illegal_chars(nickname).replace("\n", " | ")
 
     jid = JID(muc_jid)
 
