@@ -534,10 +534,14 @@ class LegacyMUC(
         form.add_field("muc#maxhistoryfetch", value=str(self.max_history_fetch))
         form.add_field("muc#roominfo_subjectmod", "boolean", value=False)
 
-        if self._ALL_INFO_FILLED_ON_STARTUP or self.stored.participants_filled:
+        if self.stored.id is not None and (
+            self._ALL_INFO_FILLED_ON_STARTUP or self.stored.participants_filled
+        ):
             with self.xmpp.store.session() as orm:
                 n = orm.scalar(
-                    sa.select(sa.func.count(Participant.id)).filter_by(room=self.stored)
+                    sa.select(sa.func.count(Participant.id)).filter_by(
+                        room_id=self.stored.id
+                    )
                 )
         else:
             n = self.n_participants
@@ -896,6 +900,8 @@ class LegacyMUC(
 
         if not self.xmpp.store.rooms.nick_available(orm, self.stored.id, nickname):
             nickname = f"{nickname} ({occupant_id})"
+            if is_user:
+                self.user_nick = nickname
 
         p = self._participant_cls(
             self,
@@ -1621,6 +1627,6 @@ def bookmarks_form():
 
 
 _BOOKMARKS_OPTIONS = bookmarks_form()
-_WHITESPACE_OR_PUNCTUATION = string.whitespace + string.punctuation
+_WHITESPACE_OR_PUNCTUATION = string.whitespace + "!\"'(),.:;?@_"
 
 log = logging.getLogger(__name__)
