@@ -307,6 +307,40 @@ class TestSession(AvatarFixtureMixin, SlidgeTest):
         for i in range(3):
             assert on_displayed.call_args_list[i][0][1] == f"msg_0{i}"
 
+    def test_mark_all_messages_duplicate(self):
+        self.xmpp.MARK_ALL_MESSAGES = True
+        self.juliet.send_text("whatever", "msg_00")
+        self.send(  # language=XML
+            """
+            <message type="chat"
+                     from="juliet@aim.shakespeare.lit/slidge"
+                     id="msg_00"
+                     to="romeo@montague.lit">
+              <body>whatever</body>
+              <active xmlns="http://jabber.org/protocol/chatstates" />
+              <store xmlns="urn:xmpp:hints" />
+              <markable xmlns="urn:xmpp:chat-markers:0" />
+            </message>
+            """,
+            use_values=False,
+        )
+        self.juliet.send_text("whatever", "msg_00")
+        self.send(  # language=XML
+            """
+            <message type="chat"
+                     from="juliet@aim.shakespeare.lit/slidge"
+                     id="msg_00"
+                     to="romeo@montague.lit">
+              <body>whatever</body>
+              <active xmlns="http://jabber.org/protocol/chatstates" />
+              <store xmlns="urn:xmpp:hints" />
+              <markable xmlns="urn:xmpp:chat-markers:0" />
+            </message>
+            """,
+            use_values=False,
+        )
+
+
     def test_movim_sticker(self):
         sticker_stanza = f"""
             <message from="romeo@montague.lit/movim"
@@ -505,7 +539,7 @@ class TestSession(AvatarFixtureMixin, SlidgeTest):
             <message type="chat"
                      from="juliet@aim.shakespeare.lit/slidge"
                      to="romeo@montague.lit"
-                     id="2">
+                     id="original-id">
               <x xmlns="jabber:x:oob">
                 <url>AVATAR_URL</url>
               </x>
@@ -518,7 +552,7 @@ class TestSession(AvatarFixtureMixin, SlidgeTest):
             """
             <message type="chat"
                      from="juliet@aim.shakespeare.lit/slidge"
-                     id="original-id"
+                     id="3"
                      to="romeo@montague.lit">
               <body>prout</body>
               <active xmlns="http://jabber.org/protocol/chatstates" />
@@ -546,15 +580,15 @@ class TestSession(AvatarFixtureMixin, SlidgeTest):
                      from="juliet@aim.shakespeare.lit/slidge"
                      id="4"
                      to="romeo@montague.lit">
-              <body>/me retracted the message 2</body>
+              <body>/me retracted the message 3</body>
               <active xmlns="http://jabber.org/protocol/chatstates" />
               <store xmlns="urn:xmpp:hints" />
               <fallback xmlns="urn:xmpp:fallback:0"
                         for="urn:xmpp:message-retract:1" />
               <retract xmlns="urn:xmpp:message-retract:1"
-                       id="2" />
+                       id="3" />
               <replace xmlns="urn:xmpp:message-correct:0"
-                       id="2" />
+                       id="3" />
             </message>
             """,
             use_values=False,
@@ -570,6 +604,8 @@ class TestSession(AvatarFixtureMixin, SlidgeTest):
                 <url>AVATAR_URL--NEW</url>
               </x>
               <body>AVATAR_URL--NEW</body>
+              <replace xmlns="urn:xmpp:message-correct:0"
+                       id="original-id" />
             </message>
             """,
             use_values=False,
@@ -584,8 +620,6 @@ class TestSession(AvatarFixtureMixin, SlidgeTest):
               <active xmlns="http://jabber.org/protocol/chatstates" />
               <markable xmlns="urn:xmpp:chat-markers:0" />
               <store xmlns="urn:xmpp:hints" />
-              <replace xmlns="urn:xmpp:message-correct:0"
-                       id="original-id" />
             </message>
             """,
             use_values=False,
@@ -705,3 +739,38 @@ class TestSession(AvatarFixtureMixin, SlidgeTest):
                 on_file.call_args.args[1]
                 == "https://download.montague.lit/4a771ac1-f0b2-4a4a-9700-f2a26fa2bb67/summit.jpg"
             )
+
+    def test_user_retract(self):
+        with unittest.mock.patch("slidge.core.session.BaseSession.on_text") as on_text:
+            on_text.return_value = "legacy-id"
+            self.recv(  # language=XML
+                """
+            <message type="chat"
+                     to="juliet@aim.shakespeare.lit"
+                     from="romeo@montague.lit/whatevs"
+                     id="origin-id">
+              <body>hoy</body>
+            </message>
+            """
+            )
+            on_text.assert_awaited_once()
+            contact = on_text.call_args[0][0]
+            text = on_text.call_args[0][1]
+            assert contact.jid.node == "juliet"
+            assert text == "hoy"
+        with unittest.mock.patch("slidge.core.session.BaseSession.on_retract") as on_retract:
+            self.recv(  # language=XML
+                """
+            <message type="chat"
+                     to="juliet@aim.shakespeare.lit"
+                     from="romeo@montague.lit/whatevs">
+              <retract id="origin-id"
+                       xmlns='urn:xmpp:message-retract:1' />
+            </message>
+            """
+            )
+            on_text.assert_awaited_once()
+            contact = on_retract.call_args[0][0]
+            legacy_msg_id = on_retract.call_args[0][1]
+            assert contact.jid.node == "juliet"
+            assert legacy_msg_id == "legacy-id"
