@@ -43,7 +43,12 @@ class Contact(LegacyContact):
 
 class MUC(LegacyMUC):
     async def update_info(self):
-        if self.legacy_id in ("room-noinfo", "room-duplicate-participant", "room-avatar-slow", "room-contact-conflict", "room-reaction-fallback"):
+        if self.legacy_id in ("room-noinfo",
+                              "room-duplicate-participant",
+                              "room-avatar-slow",
+                              "room-contact-conflict",
+                              "room-reaction-fallback",
+                              "room-mark-all-carbon"):
             return
         if self.legacy_id == "room-avatar-in-slow-task":
             self.session.create_task(self._slow_set_avatar(), "slow-avatar")
@@ -408,9 +413,6 @@ class TestSession2(AvatarFixtureMixin, SlidgeTest):
         )
         assert not muc.participants_filled
         assert self.next_sent() is None
-        muc: LegacyMUC = self.run_coro(
-            self.romeo_session.bookmarks.by_legacy_id("room-avatar-in-slow-task")
-        )
         assert not muc.participants_filled
         assert self.next_sent() is None
         self.recv(  # language=XML
@@ -607,10 +609,6 @@ class TestSession2(AvatarFixtureMixin, SlidgeTest):
         muc: LegacyMUC = self.run_coro(
             self.romeo_session.bookmarks.by_legacy_id("room-reaction-fallback")
         )
-        # yuuuuuuck, if we only call it once, then it does not have an archive PK
-        muc: LegacyMUC = self.run_coro(
-            self.romeo_session.bookmarks.by_legacy_id("room-reaction-fallback")
-        )
         part = self.run_coro(muc.get_participant_by_legacy_id("participant-x"))
         muc.add_user_resource("gajim")
         part.send_text("some text\non lines", legacy_msg_id="msg-id")
@@ -662,3 +660,203 @@ class TestSession2(AvatarFixtureMixin, SlidgeTest):
             """,
                 use_values=False,
             )
+
+    def test_muc_user_retract(self):
+        muc: LegacyMUC = self.run_coro(
+            self.romeo_session.bookmarks.by_legacy_id("room-retract")
+        )
+        muc.add_user_resource("gajim")
+        with unittest.mock.patch("slidge.core.session.BaseSession.on_text") as on_text:
+            on_text.return_value = "legacy-id"
+            self.recv(  # language=XML
+                """
+            <message type="groupchat"
+                     to="room-retract@aim.shakespeare.lit"
+                     from="romeo@montague.lit/gajim"
+                     id="origin-id">
+              <body>hoy</body>
+            </message>
+            """
+            )
+            on_text.assert_awaited_once()
+            muc = on_text.call_args[0][0]
+            text = on_text.call_args[0][1]
+            assert muc.jid.node == "room-retract"
+            assert text == "hoy"
+        self.next_sent()
+        with unittest.mock.patch("slidge.core.session.BaseSession.on_retract") as on_retract:
+            self.recv(  # language=XML
+                """
+            <message type="groupchat"
+                     to="room-retract@aim.shakespeare.lit"
+                     from="romeo@montague.lit/gajim"
+                     id="origin-id">
+              <retract id="origin-id"
+                       xmlns='urn:xmpp:message-retract:1' />
+            </message>
+            """
+            )
+            on_text.assert_awaited_once()
+            muc = on_retract.call_args[0][0]
+            legacy_msg_id = on_retract.call_args[0][1]
+            assert muc.jid.node == "room-retract"
+            assert legacy_msg_id == "legacy-id"
+
+    def test_mark_all_messages_muc(self):
+        self.xmpp.MARK_ALL_MESSAGES = True
+        muc: LegacyMUC = self.run_coro(
+            self.romeo_session.bookmarks.by_legacy_id("room-mark-all")
+        )
+        muc.add_user_resource("gajim")
+        part = self.run_coro(muc.get_participant_by_legacy_id("participant-x"))
+        part.send_text("whatever", "msg_00")
+        part.send_text("whatever", "msg_01")
+        part.send_text("whatever", "msg_02")
+        with unittest.mock.patch(
+            "slidge.core.session.BaseSession.on_displayed"
+        ) as on_displayed:
+            self.recv(  # language=XML
+                f"""
+            <message from="{self.romeo_session.user_jid.bare}/gajim"
+                     to="{muc.jid.bare}"
+                     type="groupchat">
+              <displayed xmlns='urn:xmpp:chat-markers:0'
+                         id='msg_02' />
+            </message>
+            """
+            )
+        assert on_displayed.await_count == 3
+        for i in range(3):
+            assert on_displayed.call_args_list[i][0][0].jid.username == "room-mark-all"
+            assert on_displayed.call_args_list[i][0][1] == f"msg_0{i}"
+
+        with unittest.mock.patch(
+            "slidge.core.session.BaseSession.on_displayed"
+        ) as on_displayed:
+            self.recv(  # language=XML
+                f"""
+            <message from="{self.romeo_session.user_jid.bare}/gajim"
+                     to="{muc.jid.bare}"
+                     type="groupchat">
+              <displayed xmlns='urn:xmpp:chat-markers:0'
+                         id='msg_02' />
+            </message>
+            """
+            )
+        on_displayed.assert_not_awaited()
+
+    def test_mark_all_messages_muc_not_found(self):
+        self.xmpp.MARK_ALL_MESSAGES = True
+        muc: LegacyMUC = self.run_coro(
+            self.romeo_session.bookmarks.by_legacy_id("room-mark-all")
+        )
+        muc.add_user_resource("gajim")
+        part = self.run_coro(muc.get_participant_by_legacy_id("participant-x"))
+        part.send_text("whatever", "msg_00")
+        part.send_text("whatever", "msg_01")
+        part.send_text("whatever", "msg_02")
+        with unittest.mock.patch(
+            "slidge.core.session.BaseSession.on_displayed"
+        ) as on_displayed:
+            self.recv(  # language=XML
+                f"""
+            <message from="{self.romeo_session.user_jid.bare}/gajim"
+                     to="{muc.jid.bare}"
+                     type="groupchat">
+              <displayed xmlns='urn:xmpp:chat-markers:0'
+                         id='msg_03' />
+            </message>
+            """
+            )
+        assert on_displayed.await_count == 3, on_displayed.call_args_list
+        for i in range(3):
+            assert on_displayed.call_args_list[i][0][0].jid.username == "room-mark-all"
+            assert on_displayed.call_args_list[i][0][1] == f"msg_0{i}"
+
+        with unittest.mock.patch(
+            "slidge.core.session.BaseSession.on_displayed"
+        ) as on_displayed:
+            self.recv(  # language=XML
+                f"""
+            <message from="{self.romeo_session.user_jid.bare}/gajim"
+                     to="{muc.jid.bare}"
+                     type="groupchat">
+              <displayed xmlns='urn:xmpp:chat-markers:0'
+                         id='msg_03' />
+            </message>
+            """
+            )
+        on_displayed.assert_not_awaited()
+        # assert on_displayed.call_args_list[0][0][0].jid.username == "room-mark-all"
+        # assert on_displayed.call_args_list[0][0][1] == f"msg_03"
+
+    def test_mark_all_messages_muc_carbon(self):
+        # Privileges are set as side effect of another test, when running the full test
+        # suite. I have not found where unfortunately :(
+        self.xmpp.plugin["xep_0356"].granted_privileges.clear()
+
+        self.xmpp.MARK_ALL_MESSAGES = True
+        muc: LegacyMUC = self.run_coro(
+            self.romeo_session.bookmarks.by_legacy_id("room-mark-all-carbon")
+        )
+        muc.add_user_resource("gajim")
+
+        part = self.run_coro(muc.get_participant_by_legacy_id("participant-x"))
+        part.send_text("whatever", f"msg_00")
+        _pres=self.next_sent()
+        _msg=self.next_sent()
+        for i in range(1,3):
+            part.send_text("whatever", f"msg_0{i}")
+            _msg=self.next_sent()
+
+
+        part = self.run_coro(muc.get_user_participant())
+
+        with unittest.mock.patch("uuid.uuid4", return_value="some-uuid"):
+            part.displayed("msg_01")
+            self.send(  # language=XML
+                f"""
+            <message from="{muc.jid.bare}/romeo"
+                     to="{self.romeo_session.user_jid.bare}/gajim"
+                     type="groupchat">
+              <displayed xmlns='urn:xmpp:chat-markers:0'
+                         id='msg_01' />
+              <occupant-id xmlns="urn:xmpp:occupant-id:0"
+                           id="slidge-user" />
+              <stanza-id xmlns="urn:xmpp:sid:0"
+                         id="some-uuid"
+                         by="room-mark-all-carbon@aim.shakespeare.lit" />
+            </message>
+            """
+            )
+            assert self.next_sent() is None
+        part.displayed("msg_01")
+        assert self.next_sent() is None
+        self.xmpp.MARK_ALL_MESSAGES = False
+
+    def test_mark_all_messages_muc_carbon_not_found(self):
+        self.xmpp.MARK_ALL_MESSAGES = True
+        muc = self.run_coro(
+            self.romeo_session.bookmarks.by_legacy_id("room-mark-all-carbon")
+        )
+        muc.add_user_resource("gajim")
+        part = self.run_coro(muc.get_user_participant())
+
+        with unittest.mock.patch("uuid.uuid4", return_value="some-uuid"):
+            part.displayed("msg_xx")
+            self.send(  # language=XML
+                f"""
+            <message from="{muc.jid.bare}/romeo"
+                     to="{self.romeo_session.user_jid.bare}/gajim"
+                     type="groupchat">
+              <displayed xmlns='urn:xmpp:chat-markers:0'
+                         id='msg_xx' />
+              <occupant-id xmlns="urn:xmpp:occupant-id:0"
+                           id="slidge-user" />
+              <stanza-id xmlns="urn:xmpp:sid:0"
+                         id="some-uuid"
+                         by="room-mark-all-carbon@aim.shakespeare.lit" />
+            </message>
+            """
+            )
+        self.xmpp.MARK_ALL_MESSAGES = False
